import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class AABBPhysicsDemo extends PApplet {

// Quick demo put together by Groomblecom (Groomblecom@gmail.com, groomble.com)
// If you want to use this example outside processing, then make Timer for calling draw() and replace PVector with a vector class of your own choosing. I recomend javax.vecmath for compatability reasons, but there are planty of math libraries out there.
// remember that Timers must be set to repeat.
// processing's drawing system is very similar to Graphics2D in javax.swing, so that would be my bet for a non-processing implementation.
// This implementation is far from perfect: Stacks will vibrate and such.

final float GRAVITY = 0.01f; // keep it small, because it's applied every tick- at 60 TPS.

ArrayList<Box> boxes = new ArrayList<Box>();
boolean step = false;
boolean quick = false;

public void setup() {
  size(800, 600);
  frameRate(60);
  strokeWeight(2);
  strokeCap(PROJECT);
  stroke(0);
  boxes.add(new Box(new PVector(0, 500), new PVector(2, 0))); // Vel is applied 60 times a second, here, so keep it small or beware. This demo uses discrete detection, which means that not only will things collide wierdly at high speeds, they'll even 'ghost' through.
  boxes.add(new Box(new PVector(600, 530), false)); // remember, coordinates of _upper_ _left_
  boxes.add(new Box(new PVector(400, 160)));
}

public void draw() {
  background(255);
  for(int i = 0; i < boxes.size(); i++) {
    boxes.get(i).update();
  }
  for(int i = 0; i < boxes.size(); i++) {
    boxes.get(i).satisfyConstraints();
  }
  for(int i = 0; i < boxes.size(); i++) {
    boxes.get(i).applySolution();
  }
  for(int i = 0; i < boxes.size(); i++) {
    boxes.get(i).display();
  }
}

public void mouseReleased() {
  if(mouseButton==39){
    if(!step) {noLoop();}else{loop();}
    step = !step;
    return;
  }
  if(mouseButton==3) {
    println(boxes.get(0).pos+" "+boxes.get(0).vel);
    return;
  }
  if(!step) {
    if(quick) {
      frameRate(120);
    }else{
      frameRate(60);
    }
    quick = !quick;
  }
  redraw();
}
class Box {
  PVector pos;
  PVector vel;
  PVector cornerOffset; // we'll be using pos as the upper-left, so corner-offset is the offset to the bottom-right corner.
  boolean dynamic = true; // capable of movement or not
  PVector cPos;
  PVector tD;
  Box(PVector tPos) {
    pos = tPos;
    vel = new PVector();
    cornerOffset = new PVector(50, 50);
  }
  Box(PVector tPos, PVector tVel) {
    pos = tPos;
    vel = tVel;
    cornerOffset = new PVector(50, 50);
  }
  Box(PVector tPos, boolean tDynamic) {
    pos = tPos;
    vel = null;
    dynamic = tDynamic;
    cornerOffset = new PVector(50, 50);
  }
  public void display() {
    if(dynamic) {
      fill(vel.mag()/2f*256, 100, 100);
    }else {
      fill(0, 0, 255);
    }
    rect(pos.x, pos.y, cornerOffset.x, cornerOffset.y);
  }
  public void update() {
    cPos = PVector.add(pos, cornerOffset);
    if(!dynamic) {return;}
    pos.add(vel);
    vel.y+=GRAVITY;
  }
  public void satisfyConstraints() {
    if(!dynamic) {return;}
    if(pos.x<0){pos.x=0;vel.x*=-0.999f;} // from here...
    if(pos.y<0){pos.y=0;vel.y*=-0.999f;}
    if(cPos.x>width){pos.x=width-cornerOffset.x;vel.x*=-0.999f;}
    if(cPos.y>height){pos.y=height-cornerOffset.y;vel.y*=-0.7f;if(abs(vel.y)<GRAVITY){vel.y=0;}} // to here is just keeping the boxes inside the world and a bit of hacky anti-shudder.
    tD = new PVector(pos.x, pos.y);
    for(int i = 0; i < boxes.size(); i++) { // loop through other boxes to generate contacts.
      Box b = boxes.get(i);
      if(b!=this) {
        int collisionCode = b.collisionCode(pos);  // -1 is no collision: 0 is collision in x, 1 is collision in y. This is a very simplisitic way handling it, and only works for simple simulations, like this one.
        if(collisionCode != -1) {doCollision(b, collisionCode, 51, 51);println("Collision on pos");}
        collisionCode = b.collisionCode(new PVector(pos.x, cPos.y));
        if(collisionCode != -1) {doCollision(b, collisionCode, 51, -51);println("Collision on lowX");}
        collisionCode = b.collisionCode(new PVector(cPos.x, pos.y));
        if(collisionCode != -1) {doCollision(b, collisionCode, -51, 51);println("Collision on lowY");}
        collisionCode = b.collisionCode(cPos);
        if(collisionCode != -1) {doCollision(b, collisionCode, -51, -51);println("Collision on cPos");}
      }
    }
  }
  protected void doCollision(Box b, int collisionCode, int offset0, int offset1) { // assumes code != -1
    if(b.dynamic) {
      if(collisionCode == 0) {
        float tVel = vel.x+b.vel.x;
        vel.x = tVel/2;
        b.vel.x = tVel/2;
        tD.x = b.pos.x+offset0;
      }
      if(collisionCode == 1) {
        float tVel = vel.y+b.vel.y;
        vel.y = -tVel/2;
        b.vel.y = tVel/2;
        if(abs(tD.y-b.pos.y+offset1) < 25) {
          tD.y = b.pos.y+offset1;
        }
      }
    }else{
      if(collisionCode == 0) {vel.x*=-1;tD.x = b.pos.x+offset0;}
      if(collisionCode == 1) {vel.y*=-1;print(" yFlipped ");tD.y = b.pos.y+offset1;}
    }
  }
  public int collisionCode(PVector p) {
    if(p.x>=pos.x&&p.x<=cPos.x&&p.y>=pos.y&&p.y<=cPos.y) {
      PVector rV = PVector.sub(p, pos);
      int rVal = 0;
      if(rV.x>rV.y) {
        rVal++;
      }
      print("Collision Detected: "+rVal+" ");
      if(rV.y>-rV.x+50){
        rVal++;
      }
      print(rVal+" ");
      rVal%=2;
      return rVal;
    }
    return -1;
  }
  public void applySolution() {
    if(dynamic){
      pos = tD;
    }
  }
}
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "AABBPhysicsDemo" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
