class Box {
  PVector pos;
  PVector vel;
  PVector cornerOffset; // we'll be using pos as the upper-left, so corner-offset is the offset to the bottom-right corner.
  boolean dynamic = true; // capable of movement or not
  PVector cPos;
  PVector tD;
  Box(PVector tPos) {
    pos = tPos;
    vel = new PVector();
    cornerOffset = new PVector(50, 50);
  }
  Box(PVector tPos, PVector tVel) {
    pos = tPos;
    vel = tVel;
    cornerOffset = new PVector(50, 50);
  }
  Box(PVector tPos, boolean tDynamic) {
    pos = tPos;
    vel = null;
    dynamic = tDynamic;
    cornerOffset = new PVector(50, 50);
  }
  void display() {
    if(dynamic) {
      fill(vel.mag()/2f*256, 100, 100);
    }else {
      fill(0, 0, 255);
    }
    rect(pos.x, pos.y, cornerOffset.x, cornerOffset.y);
  }
  void update() {
    cPos = PVector.add(pos, cornerOffset);
    if(!dynamic) {return;}
    pos.add(vel);
    vel.y+=GRAVITY;
  }
  void satisfyConstraints() {
    if(!dynamic) {return;}
    if(pos.x<0){pos.x=0;vel.x*=-0.999;} // from here...
    if(pos.y<0){pos.y=0;vel.y*=-0.999;}
    if(cPos.x>width){pos.x=width-cornerOffset.x;vel.x*=-0.999;}
    if(cPos.y>height){pos.y=height-cornerOffset.y;vel.y*=-0.7;if(abs(vel.y)<GRAVITY){vel.y=0;}} // to here is just keeping the boxes inside the world and a bit of hacky anti-shudder.
    tD = new PVector(pos.x, pos.y);
    for(int i = 0; i < boxes.size(); i++) { // loop through other boxes to generate contacts.
      Box b = boxes.get(i);
      if(b!=this) {
        int collisionCode = b.collisionCode(pos);  // -1 is no collision: 0 is collision in x, 1 is collision in y. This is a very simplisitic way handling it, and only works for simple simulations, like this one.
        if(collisionCode != -1) {doCollision(b, collisionCode, 51, 51);println("Collision on pos");}
        collisionCode = b.collisionCode(new PVector(pos.x, cPos.y));
        if(collisionCode != -1) {doCollision(b, collisionCode, 51, -51);println("Collision on lowX");}
        collisionCode = b.collisionCode(new PVector(cPos.x, pos.y));
        if(collisionCode != -1) {doCollision(b, collisionCode, -51, 51);println("Collision on lowY");}
        collisionCode = b.collisionCode(cPos);
        if(collisionCode != -1) {doCollision(b, collisionCode, -51, -51);println("Collision on cPos");}
      }
    }
  }
  protected void doCollision(Box b, int collisionCode, int offset0, int offset1) { // assumes code != -1
    if(b.dynamic) {
      if(collisionCode == 0) {
        float tVel = vel.x+b.vel.x;
        vel.x = tVel/2;
        b.vel.x = tVel/2;
        tD.x = b.pos.x+offset0;
      }
      if(collisionCode == 1) {
        float tVel = vel.y+b.vel.y;
        vel.y = -tVel/2;
        b.vel.y = tVel/2;
        if(abs(tD.y-b.pos.y+offset1) < 25) {
          tD.y = b.pos.y+offset1;
        }
      }
    }else{
      if(collisionCode == 0) {vel.x*=-1;tD.x = b.pos.x+offset0;}
      if(collisionCode == 1) {vel.y*=-1;print(" yFlipped ");tD.y = b.pos.y+offset1;}
    }
  }
  int collisionCode(PVector p) {
    if(p.x>=pos.x&&p.x<=cPos.x&&p.y>=pos.y&&p.y<=cPos.y) {
      PVector rV = PVector.sub(p, pos);
      int rVal = 0;
      if(rV.x>rV.y) {
        rVal++;
      }
      print("Collision Detected: "+rVal+" ");
      if(rV.y>-rV.x+50){
        rVal++;
      }
      print(rVal+" ");
      rVal%=2;
      return rVal;
    }
    return -1;
  }
  void applySolution() {
    if(dynamic){
      pos = tD;
    }
  }
}
